/***************************************************************************
                          arrayitem.cpp  -  description
                             -------------------                                         

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sat Aug 28 1999
    copyright            : (C) 2000 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <klocale.h>
#include <kapp.h>
#include <kurl.h>
#include <ksimpleconfig.h>
#include <qfileinfo.h>

#if (KDE_VERSION_MAJOR > 1) || (KDE_VERSION_MINOR > 1)
  #include <kmessagebox.h>
  #include <kio_netaccess.h>
#else
  #include <kmsgbox.h>
  #include <kfm.h>
#endif

#include "arrayitem.h"
#include "arraydlg.h"
#include "kpldoc.h"
#include "kplgraph.h"
#include "kplchecklistitem.h"
#include "utils.h"

ArrayItem::ArrayItem()
{
  init();
  ix = iy = ie = errbars = 0;
}

ArrayItem::ArrayItem(AutoStruct *aut) :  ScaledItem(aut)
{
  init();
  ix = aut->ixAuto;
  iy = aut->iyAuto;
  ie = aut->ieAuto;
  errbars = aut->autoErr;
}

ArrayItem::ArrayItem(KSimpleConfig *plo, AutoStruct *aut) : ScaledItem(plo, aut)
{
  init();
  strcpy(path, plo->readEntry("path", ""));
  if (!readFile()) {
#if (KDE_VERSION_MAJOR > 1) || (KDE_VERSION_MINOR > 1)
    KMessageBox::error(0, path);
#else
    KMsgBox::message(0, i18n("Error"), path, KMsgBox::EXCLAMATION);
#endif
    return;
  }
  int icolmax = ncols - 1;
  ix = plo->readNumEntry("ix", QMIN(aut->ixAuto, icolmax));
  iy = plo->readNumEntry("iy", QMIN(aut->iyAuto, icolmax));
  ie = plo->readNumEntry("ie", QMIN(aut->ieAuto, icolmax));
  istart = plo->readNumEntry("istart");
  n = plo->readNumEntry("n", nrows - istart);
  errbars = plo->readNumEntry("errbars", aut->autoErr);
}

ArrayItem::~ArrayItem()
{
}

itemTypes ArrayItem::iType()
{
  return ARRAYITEM;
}

void ArrayItem::init()
{
  istart = n = nrows = ncols = 0;
  x = 0;
  path[0] = 0;
}

void ArrayItem::freeX()
{
  if (x) {
    free((x)[0]);
    free(x);
    x = 0;
  }
}

void ArrayItem::freeXCond(KplDoc *m)
{
  bool del = true;
  for (KplItem *item = m->itd.first(); item; item = m->itd.next())
    if ((item != this) && (item->iType() == ARRAYITEM))
      if (((ArrayItem *) item)->x == x)
        del = false;
  if (del)
    freeX();
}

void ArrayItem::freeItem(KplDoc *m)
{
  if (x)
    freeXCond(m);
}

int ArrayItem::readFile()
{
  const char delim[] = " ,\t";
  int i, j, k, np;
  double *x1, *x2 = 0;
  char s[256], *s1;
  QString fn;
  FILE *f;

  freeX();
  ncols = nrows = 0;
  KURL u = KURL(path);
  if (u.isLocalFile())
    fn = u.path();
  else
#if (KDE_VERSION_MAJOR > 1) || (KDE_VERSION_MINOR > 1)
    if (!KIONetAccess::download(u.url(), fn))
#else
    if (!KFM::download(u.url(), fn))
#endif
      return nrows;
  if ((f = fopen(fn, "r"))) {
    do
      if ((i = (fgets(s, 256, f) != 0)))
        i = (s[0] == '#') ? 1 : 2;
    while (i == 1);
    if (i) {
      s1 = strtok(s, delim);
      while (s1) {
        ncols++;
        s1 = strtok(0, delim);
      }
      if (ncols) {
        rewind(f);
        np = 0;
        k = 0;
        while (fgets(s, 256, f))
          if (s[0] != '#') {
            nrows++;
            np += ncols;
            x2 = (double *) realloc(x2, np * sizeof(double));
            x2[k++] = ((s1 = strtok(s, delim))) ? atof(s1) : 0.0;
            for (i = 1; i < ncols; i++)
              x2[k++] = ((s1 = strtok(0, delim))) ? atof(s1) : 0.0;
          }
        x1 = (double *) malloc(np * sizeof(double));
        x = (double **) malloc(ncols * sizeof(double *));
        for (i = 0; i < ncols; i++)
          x[i] = &x1[i * nrows];
        k = 0;
        for (j = 0; j < nrows; j++)
          for (i = 0; i < ncols; i++)
            x[i][j] = x2[k++];
        free(x2);
      }
    }
    fclose(f);
  }
  if (!u.isLocalFile())
#if (KDE_VERSION_MAJOR > 1) || (KDE_VERSION_MINOR > 1)
    KIONetAccess::removeTempFile(fn);
#else
    KFM::removeTempFile(fn);
#endif
  return nrows;
}

void ArrayItem::draw(KplGraph *g)
{
  if (x && active) {
    setProperties(g);
    g->plArray(&x[ix][istart], &x[iy][istart], fx, fy, n);
    if (errbars)
      g->plError(&x[ix][istart], &x[iy][istart], &x[ie][istart], fx, fy, n);
  }
}

void ArrayItem::writePlo(FILE *f)
{
  fprintf(f, "Type=ARRAYITEM\n");
  ScaledItem::writePlo(f);
  fprintf(f, "ix=%i\niy=%i\nie=%i\nistart=%i\nn=%i\nerrbars=%i\npath=%s\n",
          ix, iy, ie, istart, n, errbars, path);
}

void ArrayItem::setText(KplCheckListItem * it, bool *arrays, bool *)
{
  QString s;
  QFileInfo fi(path);
  *arrays = true;
  it->setText(1, i18n("Array"));
  it->setText(2, s.sprintf(i18n("file %s, col. %2i %2i"),
              (const char *) fi.fileName(), ix, iy));
}

int ArrayItem::editItem(QWidget *parent, KplDoc *m, int)
{
  ArrayDlg dlg(parent, m, this);
  return dlg.exec();
}

KplItem * ArrayItem::copy()
{
  return new ArrayItem(*this);
}

void ArrayItem::expoItem(int *iext, int *ieyt, double *fxt, double *fyt)
{
  if (x) {
    expoFile(iext, fxt, &x[ix][istart], n);
    expoFile(ieyt, fyt, &x[iy][istart], n);
  }
}

void ArrayItem::minMax(double *xmi, double *xma, double *ymi, double *yma)
{
  if (x) {
    minMaxFile(xmi, xma, &x[ix][istart], n);
    minMaxFile(ymi, yma, &x[iy][istart], n);
  }
}
