/***************************************************************************
                          autodlg.cpp  -  description
                             -------------------

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sun Apr 25 1999
    copyright            : (C) 2000 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qlineedit.h>
#include <qcheckbox.h>
#include <qspinbox.h>
#include <qradiobutton.h>
#include <qbuttongroup.h>
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <klocale.h>
#include <kapp.h>
#include "autodlg.h"
#include "edittextdlg.h"
#include "symboldlg.h"
#include "utils.h"
#include "kpldoc.h"

AutoDlg::AutoDlg(QWidget *parent, KplDoc *m0) : QDialog(parent, 0, true)
{
  QString s;

  m = m0;
  a = new AutoStruct;
  *a = m->aut;
  setCaption(i18n("Autoplot settings"));
  QGroupBox *g = new QGroupBox(i18n("Window"), this);
  g->setGeometry(10, 8, 346, 102);
  QLabel *l = new QLabel(i18n("Left margin"), this);
  l->setGeometry(20, 36, 86, 22);
  autoX0 = new QLineEdit(this);
  autoX0->setGeometry(108, 36, 50, 22);
  autoX0->setText(s.setNum(a->x0Auto, 'f', 2));
  l = new QLabel("cm", this);
  l->setGeometry(166, 36, 24, 22);
  l = new QLabel(i18n("Width"), this);
  l->setGeometry(208, 36, 54, 22);
  autoXl = new QLineEdit(this);
  autoXl->setGeometry(262, 36, 50, 22);
  autoXl->setText(s.setNum(a->xlAuto, 'f', 2));
  l = new QLabel("cm", this);
  l->setGeometry(320, 36, 24, 22);
  l = new QLabel(i18n("Bottom margin"), this);
  l->setGeometry(20, 70, 86, 22);
  autoY0 = new QLineEdit(this);
  autoY0->setGeometry(108, 70, 50, 22);
  autoY0->setText(s.setNum(a->y0Auto, 'f', 2));
  l = new QLabel("cm", this);
  l->setGeometry(166, 70, 24, 22);
  l = new QLabel(i18n("Height"), this);
  l->setGeometry(208, 70, 54, 22);
  autoYl = new QLineEdit(this);
  autoYl->setGeometry(262, 70, 50, 22);
  autoYl->setText(s.setNum(a->ylAuto, 'f', 2));
  l = new QLabel("cm", this);
  l->setGeometry(320, 70, 24, 22);
  g = new QGroupBox(i18n("Labels"), this);
  g->setGeometry(10, 118, 346, 136);
  l = new QLabel(i18n("x axis"), this);
  l->setGeometry(20, 147, 64, 22);
  autoLetX = new QLineEdit(this);
  autoLetX->setGeometry(84, 147, 226, 22);
  autoLetX->setText(a->sxAuto);
  QPushButton *b = new QPushButton("...", this);
  b->setGeometry(320, 146, 25, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotEditLetX()));
  l = new QLabel(i18n("y axis"), this);
  l->setGeometry(20, 181, 64, 22);
  autoLetY = new QLineEdit(this);
  autoLetY->setGeometry(84, 181, 226, 22);
  autoLetY->setText(a->syAuto);
  b = new QPushButton("...", this);
  b->setGeometry(320, 180, 25, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotEditLetY()));
  l = new QLabel(i18n("Heading"), this);
  l->setGeometry(20, 215, 60, 22);
  autohPath = new QCheckBox(i18n("= path"), this);
  autohPath->setGeometry(80, 215, 58, 22);
  autohPath->setChecked(a->autohPath);
  connect(autohPath, SIGNAL(toggled(bool)), SLOT(disableHeading(bool)));
  autoLetH = new QLineEdit(this);
  autoLetH->setGeometry(140, 215, 170, 22);
  autoLetH->setText(a->shAuto);
  editH = new QPushButton("...", this);
  editH->setGeometry(320, 214, 25, 25);
  connect(editH, SIGNAL(clicked()), SLOT(slotEditLetH()));
  disableHeading(a->autohPath);
  bg = new QButtonGroup(i18n("Axis mode"), this);
  bg->setGeometry(370, 8, 194, 246);
  QRadioButton *r = new QRadioButton(i18n("Frame only"), bg);
  r->setGeometry(16, 38, 175, 24);
  r = new QRadioButton(i18n("Tics"), bg);
  r->setGeometry(16, 80, 175, 24);
  r = new QRadioButton(i18n("Tics with numbers"), bg);
  r->setGeometry(16, 122, 175, 24);
  r = new QRadioButton(i18n("Grid"), bg);
  r->setGeometry(16, 164, 175, 24);
  r = new QRadioButton(i18n("Grid with numbers"), bg);
  r->setGeometry(16, 206, 175, 24);
  r = (QRadioButton *) bg->find(a->autoGrid);
  r->setChecked(true);
  g = new QGroupBox(i18n("Data"), this);
  g->setGeometry(10, 260, 554, 98);
  l = new QLabel(i18n("x column"), this);
  l->setGeometry(20, 288, 64, 22);
  ix = new QSpinBox(0, 19, 1, this);
  ix->setGeometry(84, 288, 58, 22);
  ix->setValue(a->ixAuto);
  l = new QLabel(i18n("y column"), this);
  l->setGeometry(156, 288, 64, 22);
  iy = new QSpinBox(0, 19, 1, this);
  iy->setGeometry(220, 288, 58, 22);
  iy->setValue(a->iyAuto);
  l = new QLabel(i18n("Error column"), this);
  l->setGeometry(286, 288, 90, 22);
  ie = new QSpinBox(0, 19, 1, this);
  ie->setGeometry(376, 288, 58, 22);
  ie->setValue(a->ieAuto);
  ie->setEnabled(a->autoErr);
  err = new QCheckBox(i18n("Error bar"), this);
  err->setGeometry(442, 288, 112, 22);
  err->setChecked(a->autoErr);
  connect(err, SIGNAL(toggled(bool)), ie, SLOT(setEnabled(bool)));
  l = new QLabel(i18n("Symbol"), this);
  l->setGeometry(20, 322, 64, 22);
  autoSymb = new QSpinBox(-17, 9, 1, this);
  autoSymb->setGeometry(84, 322, 58, 22);
  autoSymb->setValue(a->autoSymb);
  SymbolButton *bSymb = new SymbolButton(this, autoSymb);
  bSymb->setGeometry(152, 321, 41, 25);
  autoNorm = new QCheckBox(i18n("Normalization"), this);
  autoNorm->setGeometry(220, 322, 112, 22);
  autoNorm->setChecked(a->autoNorm);
  autoFit = new QCheckBox(i18n("Autofit"), this);
  autoFit->setGeometry(376, 322, 112, 22);
  autoFit->setChecked(a->autoFit);
  b = new QPushButton(i18n("Help"), this);
  b->setGeometry(10, 366, 100, 25);
  connect(b, SIGNAL(clicked()), SLOT(slotHelp()));
  b = new QPushButton(i18n("OK"), this);
  b->setGeometry(354, 366, 100, 25);
  b->setDefault(true);
  connect(b, SIGNAL(clicked()), SLOT(slotOK()));
  b = new QPushButton(i18n("Cancel"), this);
  b->setGeometry(464, 366, 100, 25);
  connect(b, SIGNAL(clicked()), SLOT(reject()));
  adjustSize();
  setFixedSize(size());
}

AutoDlg::~AutoDlg()
{
  delete a;
}

void AutoDlg::editText(QLineEdit *ed)
{
  EditTextDlg dlg(this, ed);
  dlg.exec();
}

void AutoDlg::slotEditLetX()
{
  editText(autoLetX);
}

void AutoDlg::slotEditLetY()
{
  editText(autoLetY);
}

void AutoDlg::slotEditLetH()
{
  editText(autoLetH);
}

void AutoDlg::disableHeading(bool off)
{
  autoLetH->setEnabled(!off);
  editH->setEnabled(!off);
}

void AutoDlg::slotOK()
{
  QRadioButton *rb;

  if (!getVal(this, autoX0, &a->x0Auto, i18n("invalid position of x!")))
    return;
  if (!getVal(this, autoY0, &a->y0Auto, i18n("invalid position of y!")))
    return;
  if (!getVal(this, autoXl, &a->xlAuto, i18n("invalid width!")))
    return;
  if (!getVal(this, autoYl, &a->ylAuto, i18n("invalid height!")))
    return;
  strcpy(a->sxAuto, autoLetX->text());
  strcpy(a->syAuto, autoLetY->text());
  strcpy(a->shAuto, autoLetH->text());
  a->autohPath = autohPath->isChecked();
  a->autoFit = autoFit->isChecked();
  a->autoGrid = -1;
  do
    rb = (QRadioButton *) bg->find(++a->autoGrid);
  while (!rb->isChecked());
  a->ixAuto = ix->value();
  a->iyAuto = iy->value();
  a->ieAuto = ie->value();
  a->autoErr = err->isChecked();
  a->autoSymb = autoSymb->value();
  a->autoNorm = autoNorm->isChecked();
  m->aut = *a;
  accept();
}

void AutoDlg::slotHelp()
{
  kapp->invokeHTMLHelp("kpl/index-5.html", "sec-autoplot");
}
