/***************************************************************************
                          funitem.cpp  -  description
                             -------------------                                         

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sun Aug 29 1999
    copyright            : (C) 2000 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <string.h>
#include <dlfcn.h>
#include <kapp.h>
#include <klocale.h>
#include <ksimpleconfig.h>
#include <qfileinfo.h>
#include "funitem.h"
#include "funcdlg.h"
#include "kplgraph.h"
#include "kplchecklistitem.h"
#include "utils.h"

FunItem::FunItem()
{
  init();
}

FunItem::FunItem(AutoStruct *aut) : ScaledItem(aut)
{
  init();
}

FunItem::FunItem(KSimpleConfig *plo, AutoStruct *aut) : ScaledItem(plo, aut)
{
  char s1[256];
  QString s2;

  fkty = 0;
  hmody = 0;
  tv = yv = 0;
  tmin = plo->readDoubleNumEntry("xfmin");
  tmax = plo->readDoubleNumEntry("xfmax", tmin + 1.0);
  dt = plo->readDoubleNumEntry("dx");
  strcpy(s1, plo->readEntry("p", "0"));
  py[0] = ((s2 = strtok(s1, " "))) ? s2.toDouble() : 0.0;
  for (int j = 1; j < NPMAX; j++)
    py[j] = ((s2 = strtok(0, " "))) ? s2.toDouble() : 0.0;
  strcpy(namey, plo->readEntry("name", ""));
  strcpy(pathy, plo->readEntry("path", ""));
  getFuncAddr(pathy, namey, &hmody, &fkty);
}

FunItem::~FunItem()
{
}

itemTypes FunItem::iType()
{
  return FUNITEM;
}

void FunItem::init()
{
  tmin = tmax = dt = 0.0;
  fkty = 0;
  memset(py, 0, sizeof(py));
  namey[0] = pathy[0] = 0;
  hmody = 0;
  tv = yv = 0;
}

void FunItem::freeItem(KplDoc *)
{
  if (hmody)
    dlclose(hmody);
  if (tv)
    delete tv;
  if (yv)
    delete yv;
}

void FunItem::draw(KplGraph *g)
{
  int i, n;

  if (fkty && active) {
    setProperties(g);
    bool newv = !tv;
    double d = dt;
    if (d)
      n = int(fabs((tmax - tmin) / d)) + 1;
    else {
      d = 0.01 * (tmax - tmin);
      n = 101;
    }
    if (newv || (tmin != tmino) || (tmax != tmaxo) || (d != dto)) {
      tv = (double *) realloc(tv, n * sizeof(double));
      for (i = 0; i < n; i++)
        tv[i] = tmin + i * d;
    }
    bool pchanged = false;
    for (i = 0; i < NPMAX; i++)
      if ((pchanged = (py[i] != pyo[i])))
        break;
    if (pchanged || newv || (tmin != tmino) || (tmax != tmaxo) ||
        (d != dto) || (fkty != fktyo))
      yv = (double *) realloc(yv, n * sizeof(double));
    if (pchanged || newv || (tmin != tmino) || (tmax > tmaxo) ||
        (d != dto) || (fkty != fktyo)) {
      for (i = 0; i < n; i++)
        yv[i] = ((*fkty)(tv[i], py));
    }
    if (pchanged)
      for (i = 0; i < NPMAX; i++)
        pyo[i] = py[i];
    if (fkty != fktyo)
      fktyo = fkty;
    if (tmin != tmino)
      tmino = tmin;
    if (tmax != tmaxo)
      tmaxo = tmax;
    if (d != dto)
      dto = d;
    g->plArray(tv, yv, fx, fy, n);
  }
}

void FunItem::writePlo(FILE *f)
{
  fprintf(f, "Type=FUNITEM\n");
  ScaledItem::writePlo(f);
  fprintf(f, "xfmin=%g\nxfmax=%g\ndx=%g\np=", tmin, tmax, dt);
  for (int i = 0; i < 19; i++)
    fprintf(f, "%g ", py[i]);
  fprintf(f, "%g\nname=%s\npath=%s\n", py[19], namey, pathy);
}

void FunItem::setText(KplCheckListItem * it, bool *, bool *funcs)
{
  QString s;
  QFileInfo fi(pathy);
  *funcs = true;
  it->setText(1, i18n("Function"));
  it->setText(2, s.sprintf("%s.%s", (const char *) fi.baseName(), namey));
}

int FunItem::editItem(QWidget *parent, KplDoc *m, int)
{
  FuncDlg dlg(parent, m, this);
  return dlg.exec();
}

KplItem * FunItem::copy()
{
  FunItem *it = new FunItem(*this);
  if (hmody)
    it->hmody = dlopen(it->pathy, RTLD_NOW);
  if (tv)
    tv = 0;
  if (yv)
    yv = 0;
  return it;
}

void FunItem::expoItem(int *iext, int *ieyt, double *fxt, double *fyt)
{
  if (fkty) {
    expo(QMAX(fabs(tmin), fabs(tmax)), iext, fxt);
    expoFunc(fkty, ieyt, fyt, tmin, tmax, dt, py);
  }
}

void FunItem::minMax(double *xmi, double *xma, double *ymi, double *yma)
{
  int mtic, ndig;
  double ytic;

  if (fkty) {
    *xmi = tmin;
    *xma = tmax;
    autoFunc(ymi, yma, &ytic, &mtic, &ndig, fkty, *xmi, *xma, dt, py, 1.0, 0.0);
  }
}

ParFunItem::ParFunItem()
{
  init();
}

ParFunItem::ParFunItem(AutoStruct *aut) :
 FunItem(aut)
{
  init();
}

ParFunItem::ParFunItem(KSimpleConfig *plo, AutoStruct *aut)
{
  char s1[256];
  QString s2;

  fktx = 0;
  hmodx = 0;
  xv = 0;
  symb = plo->readNumEntry("symb", aut->autoSymb);
  strcpy(s1, plo->readEntry("color", ""));
  if (strlen(s1))
    sscanf(s1, "%lx", &color);
  else
    color = aut->colData;
  fx = plo->readDoubleNumEntry("fx", 1.0);
  fy = plo->readDoubleNumEntry("fy", 1.0);
  tmin = plo->readDoubleNumEntry("tmin");
  tmax = plo->readDoubleNumEntry("tmax", tmin + 1.0);
  dt = plo->readDoubleNumEntry("dt");
  strcpy(s1, plo->readEntry("px", "0"));
  px[0] = ((s2 = strtok(s1, " "))) ? s2.toDouble() : 0.0;
  for (int j = 1; j < NPMAX; j++)
    px[j] = ((s2 = strtok(0, " "))) ? s2.toDouble() : 0.0;
  strcpy(s1, plo->readEntry("py", "0"));
  py[0] = ((s2 = strtok(s1, " "))) ? s2.toDouble() : 0.0;
  for (int j = 1; j < NPMAX; j++)
    py[j] = ((s2 = strtok(0, " "))) ? s2.toDouble() : 0.0;
  strcpy(namex, plo->readEntry("namex", ""));
  strcpy(namey, plo->readEntry("namey", ""));
  strcpy(pathx, plo->readEntry("pathx", ""));
  strcpy(pathy, plo->readEntry("pathy", ""));
  active = plo->readBoolEntry("active", true);
  getFuncAddr(pathx, namex, &hmodx, &fktx);
  getFuncAddr(pathy, namey, &hmody, &fkty);
}

ParFunItem::~ParFunItem()
{
}

itemTypes ParFunItem::iType()
{
  return PARFUNITEM;
}

void ParFunItem::init()
{
  fktx = 0;
  memset(px, 0, sizeof(px));
  namex[0] = pathx[0] = 0;
  hmodx = 0;
  xv = 0;
}

void ParFunItem::freeItem(KplDoc *)
{
  if (hmodx)
    dlclose(hmodx);
  if (hmody)
    dlclose(hmody);
  if (xv)
    delete xv;
}

void ParFunItem::draw(KplGraph *g)
{
  int i, n;
  bool pchanged = false;

  if (fkty && active) {
    setProperties(g);
    bool newv = !tv;
    double d = dt;
    if (d)
      n = int(fabs((tmax - tmin) / d)) + 1;
    else {
      d = 0.01 * (tmax - tmin);
      n = 101;
    }
    if (newv || (tmin != tmino) || (tmax != tmaxo) || (d != dto)) {
      tv = (double *) realloc(tv, n * sizeof(double));
      for (i = 0; i < n; i++)
        tv[i] = tmin + i * d;
    }
    for (i = 0; i < NPMAX; i++)
      if ((pchanged = (py[i] != pyo[i])))
        break;
    if (pchanged || newv || (tmin != tmino) || (tmax != tmaxo) ||
        (d != dto) || (fkty != fktyo))
      yv = (double *) realloc(yv, n * sizeof(double));
    if (pchanged || newv || (tmin != tmino) || (tmax > tmaxo) ||
        (d != dto) || (fkty != fktyo)) {
      for (i = 0; i < n; i++)
        yv[i] = ((*fkty)(tv[i], py));
    }
    if (pchanged) {
      for (i = 0; i < NPMAX; i++)
        pyo[i] = py[i];
      pchanged = false;
    }
    for (i = 0; i < NPMAX; i++)
      if ((pchanged = (px[i] != pxo[i])))
        break;
    if (pchanged || newv || (tmin != tmino) || (tmax != tmaxo) ||
        (d != dto) || (fktx != fktxo))
      xv = (double *) realloc(xv, n * sizeof(double));
    if (pchanged || newv || (tmin != tmino) || (tmax > tmaxo) ||
        (d != dto) || (fktx != fktxo)) {
      for (i = 0; i < n; i++)
        xv[i] = ((*fktx)(tv[i], px));
    }
    if (pchanged)
      for (i = 0; i < NPMAX; i++)
        pxo[i] = px[i];
    if (fktx != fktxo)
      fktxo = fktx;
    if (fkty != fktyo)
      fktyo = fkty;
    if (tmin != tmino)
      tmino = tmin;
    if (tmax != tmaxo)
      tmaxo = tmax;
    if (d != dto)
      dto = d;
    g->plArray(xv, yv, fx, fy, n);
  }
}

void ParFunItem::writePlo(FILE *f)
{
  int i;

  fprintf(f, "Type=PARFUNITEM\n");
  ScaledItem::writePlo(f);
  fprintf(f, "tmin=%g\ntmax=%g\ndt=%g\npx=", tmin, tmax, dt);
  for (i = 0; i < 19; i++)
    fprintf(f, "%g ", px[i]);
  fprintf(f, "%g\npy=", px[19]);
  for (i = 0; i < 19; i++)
    fprintf(f, "%g ", py[i]);
  fprintf(f, "%g\nnamex=%s\nnamey=%s\npathx=%s\npathy=%s\n",
          py[19], namex, namey, pathx, pathy);
}

void ParFunItem::setText(KplCheckListItem * it, bool *, bool *)
{
  QString s, s2;
  QFileInfo fi(pathx);
  it->setText(1, i18n("Par. function"));
  it->setText(2, s.sprintf("%s.%s", (const char *) fi.baseName(), namey));
  s2.sprintf("x %s.%s", (const char *) fi.baseName(), namex);
  fi.setFile(pathy);
  it->setText(2, s.sprintf("%s, y %s.%s",
              (const char *) s2, (const char *) fi.baseName(), namey));
}

int ParFunItem::editItem(QWidget *parent, KplDoc *m, int)
{
  ParFuncDlg dlg(parent, m, this);
  return dlg.exec();
}

KplItem * ParFunItem::copy()
{
  ParFunItem *it = new ParFunItem(*this);
  if (hmodx)
    it->hmodx = dlopen(it->pathx, RTLD_NOW);
  if (hmody)
    it->hmody = dlopen(it->pathy, RTLD_NOW);
  if (yv)
    yv = 0;
  return it;
}

void ParFunItem::expoItem(int *iext, int *ieyt, double *fxt, double *fyt)
{
  if (fktx && fkty) {
    expoFunc(fktx, iext, fxt, tmin, tmax, dt, px);
    expoFunc(fkty, ieyt, fyt, tmin, tmax, dt, py);
  }
}

void ParFunItem::minMax(double *xmi, double *xma, double *ymi, double *yma)
{
  int mtic, ndig;
  double tic;

  if (fktx && fkty) {
    autoFunc(xmi, xma, &tic, &mtic, &ndig, fktx, tmin, tmax, dt, px, 1.0, 0.0);
    autoFunc(ymi, yma, &tic, &mtic, &ndig, fkty, tmin, tmax, dt, py, 1.0, 0.0);
  }
}
