/***************************************************************************
                          funitem.h  -  description
                             -------------------                                         

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sun Aug 29 1999
    copyright            : (C) 2000 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#ifndef FUNITEM_H
#define FUNITEM_H

#include <kplitem.h>

class KSimpleConfig;
class KplCheckListItem;

/**
  * Function item class. Class for function items showing ordinary functions
  * by lines or markers.
  * @author Werner Stille
  */

class FunItem : public ScaledItem  {

public:
  /** Constructor */
	FunItem();
  /**
    * Constructor. Initializes corresponding to autoplot settings.
    * @param aut pointer to structure containing autoplot settings.
    */
  FunItem(AutoStruct *aut);
  /**
    * Constructor. Initializes corresponding to plot file.
    * @param plo pointer to KSimpleConfig object containing item data.
    * @param aut pointer to structure containing autoplot settings.
    */
  FunItem(KSimpleConfig *plo, AutoStruct *aut);
  /** Destructor */
  virtual ~FunItem();
  virtual itemTypes iType();
  /**
   * Closes access to dynamic loaded library functions.
   * @param m pointer to KplDoc model object.
   */
  virtual void freeItem(KplDoc *m);
  virtual void draw(KplGraph *g);
  virtual void writePlo(FILE *f);
  virtual void setText(KplCheckListItem *it, bool *arrays, bool *funcs);
  virtual int editItem(QWidget *parent, KplDoc *m, int i);
  virtual KplItem* copy();
  virtual void expoItem(int *iext, int *ieyt, double *fxt, double *fyt);
  virtual void minMax(double *xmi, double *xma, double *ymi, double *yma);

protected:
  /**
   * Initializes several elements.
   */
  void init();
  double tmin, tmax, dt, (*fkty)(double, double *), py[NPMAX], *tv, *yv,
         tmino, tmaxo, dto, (*fktyo)(double, double *), pyo[NPMAX];
  char namey[100], pathy[260];
  void *hmody;

  friend class FitDlg;
  friend class FuncDlg;
  friend class KplDoc;
};

/**
  * Parametric function item class.
  * @author Werner Stille
  */

class ParFunItem : public FunItem {

public:
  /** Constructor */
  ParFunItem();
  /**
    * Constructor. Initializes corresponding to autoplot settings.
    * @param aut pointer to structure containing autoplot settings.
    */
  ParFunItem(AutoStruct *aut);
  /**
    * Constructor. Initializes corresponding to plot file.
    * @param plo pointer to KSimpleConfig object containing item data.
    * @param aut pointer to structure containing autoplot settings.
    */
  ParFunItem(KSimpleConfig *plo, AutoStruct *aut);
  /** Destructor */
  virtual ~ParFunItem();
  virtual itemTypes iType();
  virtual void freeItem(KplDoc *m);
  virtual void draw(KplGraph *g);
  virtual void writePlo(FILE *f);
  virtual void setText(KplCheckListItem *it, bool *arrays, bool *funcs);
  virtual int editItem(QWidget *parent, KplDoc *m, int i);
  virtual KplItem* copy();
  virtual void expoItem(int *iext, int *ieyt, double *fxt, double *fyt);
  virtual void minMax(double *xmi, double *xma, double *ymi, double *yma);

protected:
  /**
   * Initializes several elements.
   */
  void init();
  double (*fktx)(double, double *), px[NPMAX], *xv,
         (*fktxo)(double, double *), pxo[NPMAX];
  char namex[100], pathx[260];
  void *hmodx;
  friend class ParFuncDlg;
};

#endif
