/***************************************************************************
                          kpl.cpp  -  description
                             -------------------

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sat Apr 24 15:14:00 MEST 1999

    copyright            : (C) 2000 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include <math.h>
#include <qprinter.h>
#include <qspinbox.h>
#include <qscrollbar.h>
#include <qimage.h>
#include <qdir.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kurl.h>
#include <kfiledialog.h>
#include <kaccel.h>
#include <kprocess.h>
#include <kkeydialog.h>
#include <kmenubar.h>
#include <kconfig.h>
#include <kapp.h>

#if (KDE_VERSION_MAJOR > 1) || (KDE_VERSION_MINOR > 1)
  #include <kmessagebox.h>
  #include <kio_job.h>
  #include <kglobal.h>
  #include <kinstance.h>
  #include <kstddirs.h>
#else
  #include <kmsgbox.h>
  #include <kfm.h>
#endif

#include "kpl.h"
#include "kpldoc.h"
#include "kplview.h"
#include "kplwidget.h"
#include "resource.h"
#include "zoomdlg.h"
#include "refreshdlg.h"
#include "basicdlg.h"
#include "autodlg.h"
#include "itemdlg.h"
#include "autofitdlg.h"
#include "psgraph.h"
#include "kplitem.h"
#include "utils.h"

KplApp::KplApp()
{
#if (KDE_VERSION_MAJOR > 1) || (KDE_VERSION_MINOR > 1)
  config = KGlobal::config();
#else
  config = kapp->getConfig();
#endif
  plot = false;
  itemDlg = 0;
  tRefresh = 5;
  setMinimumSize(120, 80);
  popupMenu = new QPopupMenu();
  initMenuBar();
  initToolBar();
  initDocument();
  readOptions();
  initView();
  calcStatusBar();
  initStatusBar();
  initKeyAccel();
  updWin();
  timer = new QTimer(this);
  connect(timer, SIGNAL(timeout()), SLOT(slotViewReload()));
  connect(kapp, SIGNAL(lastWindowClosed()), kapp, SLOT(quit()));
#if (KDE_VERSION_MAJOR < 2) && (KDE_VERSION_MINOR < 2)
  connect(new KDNDDropZone(this, DndURL),
          SIGNAL(dropAction(KDNDDropZone *)),
          SLOT(slotDropEvent(KDNDDropZone *)));
#endif
  connect(doc, SIGNAL(modelChanged()), SLOT(slotUpdate()));
  connect(view->page, SIGNAL(mouseMoved(int, int)), SLOT(updStatus(int, int)));
  connect(view->page, SIGNAL(mouseLeaved()), SLOT(clearStatus()));
  connect(view->page, SIGNAL(rightButtonPressed()), SLOT(showPopupMenu()));
  connect(view->page, SIGNAL(urlsDropped(QStrList *)), SLOT(slotOpenList(QStrList *)));
  connect(popupMenu, SIGNAL(highlighted(int)), SLOT(statusCallback(int)));
}

KplApp::~KplApp()
{
  delete key_accel;
  delete fileMenu;
  delete viewMenu;
  delete optionsMenu;
  delete recentFilesMenu;
  delete popupMenu;
  if (itemDlg)
    delete itemDlg;
}

bool KplApp::openDocumentFile(const char* path)
{
  statusBar()->message(i18n("Opening file..."));
	bool rc = doc->openDocument(path);
  if (itemDlg)
    itemDlg->updItemList();
  slotStatusMsg(i18n("Ready."));
  return rc;
}

KplDoc* KplApp::getDocument() const
{
	return doc;
}

void KplApp::initMenuBar()
{
  recentFilesMenu = new QPopupMenu();
  connect(recentFilesMenu, SIGNAL(activated(int)), SLOT(slotFileOpenRecent(int)));
  fileMenu = new QPopupMenu();
#if (KDE_VERSION_MAJOR > 1) || (KDE_VERSION_MINOR > 1)
  fileMenu->insertItem(BarIcon("filenew"), i18n("&New Plot..."), this,
                       SLOT(slotFileNew()), 0, ID_FILE_NEW);
  fileMenu->insertItem(BarIcon("fileopen"), i18n("&Open Plot File..."), this,
                       SLOT(slotFileOpenPlot()), 0, ID_FILE_OPEN_PLOT);
  fileMenu->insertItem(BarIcon("fileopen"), i18n("Open &Data File..."), this,
                       SLOT(slotFileOpenData()), 0, ID_FILE_OPEN_DATA);
  fileMenu->insertItem(i18n("Open &Recent"), recentFilesMenu, ID_FILE_OPEN_RECENT);
  fileMenu->insertSeparator();
  fileMenu->insertItem(BarIcon("filefloppy") ,i18n("&Save"), this,
                       SLOT(slotFileSave()), 0, ID_FILE_SAVE);
  fileMenu->insertItem(i18n("Save &As..."), this, SLOT(slotFileSaveAs()),
                       0, ID_FILE_SAVE_AS);
  fileMenu->insertItem(i18n("&Close"), this, SLOT(slotFileClose()),
                       0, ID_FILE_CLOSE);
  fileMenu->insertSeparator();
  fileMenu->insertItem(i18n("Display P&lot"), this, SLOT(slotFilePlot()),
                       0, ID_FILE_PLOT);
  fileMenu->insertItem(i18n("PostScript O&utput..."), this, SLOT(slotFilePS()),
                       0, ID_FILE_PS);
  fileMenu->insertSeparator();
  fileMenu->insertItem(BarIcon("fileprint"), i18n("&Print..."), this,
                       SLOT(slotFilePrint()), 0, ID_FILE_PRINT);
  fileMenu->insertSeparator();
  fileMenu->insertItem(kapp->miniIcon(), i18n("New &Window"), this,
                       SLOT(slotFileNewWindow()), 0, ID_FILE_NEW_WINDOW);
  fileMenu->insertItem(i18n("Close W&indow"), this, SLOT(slotFileCloseWindow()),
                       0, ID_FILE_CLOSE_WINDOW);
  fileMenu->insertSeparator();
  fileMenu->insertItem(i18n("E&xit"), this, SLOT(slotFileQuit()),
                       0, ID_FILE_QUIT);
  viewMenu = new QPopupMenu();
  viewMenu->setCheckable(true);
  viewMenu->insertItem(BarIcon("viewmag+"), i18n("Zoom &In"), this,
                       SLOT(slotViewZoomIn()), 0, ID_VIEW_ZOOM_IN);
  viewMenu->insertItem(BarIcon("viewmag-"), i18n("Zoom &Out"), this,
                       SLOT(slotViewZoomOut()), 0, ID_VIEW_ZOOM_OUT);
  viewMenu->insertItem(BarIcon("viewzoom"), i18n("&Zoom Factor..."), this,
                       SLOT(slotViewZoom()), 0, ID_VIEW_ZOOM);
  viewMenu->insertSeparator();
  viewMenu->insertItem(BarIcon("reload"), i18n("&Reload"), this,
                       SLOT(slotViewReload()), 0, ID_VIEW_RELOAD);
  viewMenu->insertItem(i18n("Reload &Periodically..."), this,
                       SLOT(slotViewAutoReload()), 0, ID_VIEW_AUTO_RELOAD);
  optionsMenu = new QPopupMenu();
  optionsMenu->setCheckable(true);
  optionsMenu->insertItem(i18n("Show &Tool Bar"), this,
                          SLOT(slotOptionsToolBar()), 0, ID_OPTIONS_TOOLBAR);
  optionsMenu->insertItem(i18n("Show Stat&us Bar"), this,
                          SLOT(slotOptionsStatusBar()), 0,
                          ID_OPTIONS_STATUSBAR);
  optionsMenu->insertItem(i18n("&Autoplot"), this,
                          SLOT(slotOptionsAutoPlot()), 0, ID_OPTIONS_AUTOPLOT);
  optionsMenu->insertItem(i18n("S&how Function Source"), this,
                          SLOT(slotOptionsShowSource()), 0,
                          ID_OPTIONS_SHOWSOURCE);
  optionsMenu->insertItem(i18n("Calculate &PS Bounding Box"), this,
                          SLOT(slotOptionsBBox()), 0, ID_OPTIONS_BBOX);
  optionsMenu->insertItem(i18n("Unsaved Changes &Warning"), this,
                          SLOT(slotOptionsWarning()), 0, ID_OPTIONS_WARNING);
  optionsMenu->insertItem(i18n("Save Options At &End"), this,
                          SLOT(slotOptionsSaveAtEnd()), 0,
                          ID_OPTIONS_SAVE_AT_END);
  optionsMenu->insertSeparator();
  optionsMenu->insertItem(i18n("Configure &Keys Bindings..."), this,
                          SLOT(slotOptionsKeys()), 0, ID_OPTIONS_KEYS);
  optionsMenu->insertSeparator();
  optionsMenu->insertItem(i18n("Configure &Basics..."), this,
                          SLOT(slotOptionsBasic()), 0, ID_OPTIONS_BASIC);
  optionsMenu->insertItem(i18n("&Configure Autoplot..."), this,
                          SLOT(slotOptionsAuto()), 0, ID_OPTIONS_AUTO);
  optionsMenu->insertItem(i18n("Configure Auto&fit..."), this,
                          SLOT(slotOptionsAutoFit()), 0, ID_OPTIONS_AUTOFIT);
  optionsMenu->insertSeparator();
  optionsMenu->insertItem(i18n("&Save Options"), this,
                          SLOT(slotOptionsSave()), 0, ID_OPTIONS_SAVE);
  menuBar()->insertItem(i18n("&File"), fileMenu);
  menuBar()->insertItem(i18n("&Edit"), this, SLOT(slotEditItems()));
  menuBar()->insertItem(i18n("&View"), viewMenu);
  menuBar()->insertItem(i18n("&Options"), optionsMenu);
  menuBar()->insertSeparator();
  menuBar()->insertItem(i18n("&Help"), helpMenu(i18n(IDS_APP_ABOUT), false));
#else
  fileMenu->insertItem(Icon("filenew.xpm"), i18n("&New Plot..."), this,
                       SLOT(slotFileNew()), 0, ID_FILE_NEW);
  fileMenu->insertItem(Icon("fileopen.xpm"), i18n("&Open Plot File..."), this,
                       SLOT(slotFileOpenPlot()), 0, ID_FILE_OPEN_PLOT);
  fileMenu->insertItem(Icon("fileopen.xpm"), i18n("Open &Data File..."), this,
                       SLOT(slotFileOpenData()), 0, ID_FILE_OPEN_DATA);
  fileMenu->insertItem(i18n("Open &Recent"), recentFilesMenu, ID_FILE_OPEN_RECENT);
  fileMenu->insertSeparator();
  fileMenu->insertItem(Icon("filefloppy.xpm") ,i18n("&Save"), this,
                       SLOT(slotFileSave()), 0, ID_FILE_SAVE);
  fileMenu->insertItem(i18n("Save &As..."), this, SLOT(slotFileSaveAs()),
                       0, ID_FILE_SAVE_AS);
  fileMenu->insertItem(i18n("&Close"), this, SLOT(slotFileClose()),
                       0, ID_FILE_CLOSE);
  fileMenu->insertSeparator();
  fileMenu->insertItem(i18n("Display P&lot"), this, SLOT(slotFilePlot()),
                       0, ID_FILE_PLOT);
  fileMenu->insertItem(i18n("PostScript O&utput..."), this, SLOT(slotFilePS()),
                       0, ID_FILE_PS);
  fileMenu->insertSeparator();
  fileMenu->insertItem(Icon("fileprint.xpm"), i18n("&Print..."), this,
                       SLOT(slotFilePrint()), 0, ID_FILE_PRINT);
  fileMenu->insertSeparator();
  fileMenu->insertItem(kapp->getMiniIcon(), i18n("New &Window"), this,
                       SLOT(slotFileNewWindow()), 0, ID_FILE_NEW_WINDOW);
  fileMenu->insertItem(i18n("Close W&indow"), this, SLOT(slotFileCloseWindow()),
                       0, ID_FILE_CLOSE_WINDOW);
  fileMenu->insertSeparator();
  fileMenu->insertItem(i18n("E&xit"), this, SLOT(slotFileQuit()),
                       0, ID_FILE_QUIT);
  viewMenu = new QPopupMenu();
  viewMenu->setCheckable(true);
  viewMenu->insertItem(Icon("viewmag+.xpm"), i18n("Zoom &In"), this,
                       SLOT(slotViewZoomIn()), 0, ID_VIEW_ZOOM_IN);
  viewMenu->insertItem(Icon("viewmag-.xpm"), i18n("Zoom &Out"), this,
                       SLOT(slotViewZoomOut()), 0, ID_VIEW_ZOOM_OUT);
  viewMenu->insertItem(Icon("viewzoom.xpm"), i18n("&Zoom Factor..."), this,
                       SLOT(slotViewZoom()), 0, ID_VIEW_ZOOM);
  viewMenu->insertSeparator();
  viewMenu->insertItem(Icon("reload.xpm"), i18n("&Reload"), this,
                       SLOT(slotViewReload()), 0, ID_VIEW_RELOAD);
  viewMenu->insertItem(i18n("Reload &Periodically..."), this,
                       SLOT(slotViewAutoReload()), 0, ID_VIEW_AUTO_RELOAD);
  optionsMenu = new QPopupMenu();
  optionsMenu->setCheckable(true);
  optionsMenu->insertItem(i18n("Show &Tool Bar"), this,
                          SLOT(slotOptionsToolBar()), 0, ID_OPTIONS_TOOLBAR);
  optionsMenu->insertItem(i18n("Show Stat&us Bar"), this,
                          SLOT(slotOptionsStatusBar()), 0,
                          ID_OPTIONS_STATUSBAR);
  optionsMenu->insertItem(i18n("&Autoplot"), this,
                          SLOT(slotOptionsAutoPlot()), 0, ID_OPTIONS_AUTOPLOT);
  optionsMenu->insertItem(i18n("S&how Function Source"), this,
                          SLOT(slotOptionsShowSource()), 0,
                          ID_OPTIONS_SHOWSOURCE);
  optionsMenu->insertItem(i18n("Calculate &PS Bounding Box"), this,
                          SLOT(slotOptionsBBox()), 0, ID_OPTIONS_BBOX);
  optionsMenu->insertItem(i18n("Unsaved Changes &Warning"), this,
                          SLOT(slotOptionsWarning()), 0, ID_OPTIONS_WARNING);
  optionsMenu->insertItem(i18n("Save Options At &End"), this,
                          SLOT(slotOptionsSaveAtEnd()), 0,
                          ID_OPTIONS_SAVE_AT_END);
  optionsMenu->insertSeparator();
  optionsMenu->insertItem(i18n("Configure &Keys Bindings..."), this,
                          SLOT(slotOptionsKeys()), 0, ID_OPTIONS_KEYS);
  optionsMenu->insertSeparator();
  optionsMenu->insertItem(i18n("Configure &Basics..."), this,
                          SLOT(slotOptionsBasic()), 0, ID_OPTIONS_BASIC);
  optionsMenu->insertItem(i18n("&Configure Autoplot..."), this,
                          SLOT(slotOptionsAuto()), 0, ID_OPTIONS_AUTO);
  optionsMenu->insertItem(i18n("Configure Auto&fit..."), this,
                          SLOT(slotOptionsAutoFit()), 0, ID_OPTIONS_AUTOFIT);
  optionsMenu->insertSeparator();
  optionsMenu->insertItem(i18n("&Save Options"), this,
                          SLOT(slotOptionsSave()), 0, ID_OPTIONS_SAVE);
  menuBar()->insertItem(i18n("&File"), fileMenu);
  menuBar()->insertItem(i18n("&Edit"), this, SLOT(slotEditItems()));
  menuBar()->insertItem(i18n("&View"), viewMenu);
  menuBar()->insertItem(i18n("&Options"), optionsMenu);
  menuBar()->insertSeparator();
  menuBar()->insertItem(i18n("&Help"),
                             kapp->getHelpMenu(true, i18n(IDS_APP_ABOUT)));
#endif
  connect(menuBar(), SIGNAL(highlighted(int)), SLOT(statusCallback(int)));
}

void KplApp::initToolBar()
{
#if (KDE_VERSION_MAJOR > 1) || (KDE_VERSION_MINOR > 1)
  toolBar()->insertButton(BarIcon("filenew"), ID_FILE_NEW,
                          SIGNAL(clicked()), this, SLOT(slotFileNew()),
                          true, i18n("New Plot..."));
  toolBar()->insertButton(BarIcon("fileopen"), ID_FILE_OPEN_PLOT,
                          SIGNAL(clicked()), this, SLOT(slotFileOpenPlot()),
                          true, i18n("Open Plot File..."));
  toolBar()->insertButton(BarIcon("fileopen"), ID_FILE_OPEN_DATA,
                          SIGNAL(clicked()), this, SLOT(slotFileOpenData()),
                          true, i18n("Open Data File..."));
  toolBar()->insertSeparator();
  toolBar()->insertButton(BarIcon("filefloppy"), ID_FILE_SAVE,
                          SIGNAL(clicked()), this, SLOT(slotFileSave()),
                          true, i18n("Save"));
  toolBar()->insertSeparator();
  toolBar()->insertButton(BarIcon("fileprint"), ID_FILE_PRINT,
                          SIGNAL(clicked()), this, SLOT(slotFilePrint()),
                          true, i18n("Print..."));
  toolBar()->insertSeparator();
  toolBar()->insertButton(BarIcon("viewmag+"), ID_VIEW_ZOOM_IN,
                          SIGNAL(clicked()), this, SLOT(slotViewZoomIn()),
                          true, i18n("Zoom In"));
  toolBar()->insertButton(BarIcon("viewmag-"), ID_VIEW_ZOOM_OUT,
                          SIGNAL(clicked()), this, SLOT(slotViewZoomOut()),
                          true, i18n("Zoom Out"));
  toolBar()->insertSeparator();
  toolBar()->insertButton(BarIcon("reload"), ID_VIEW_RELOAD,
                          SIGNAL(clicked()), this, SLOT(slotViewReload()),
                          true, i18n("Reload"));
  toolBar()->insertSeparator();
  toolBar()->insertButton(BarIcon("help"), ID_HELP,
                          SIGNAL(clicked()), this, SLOT(appHelpActivated()),
                          true, i18n("Help"));
#else
  toolBar()->insertButton(Icon("filenew.xpm"), ID_FILE_NEW,
                          SIGNAL(clicked()), this, SLOT(slotFileNew()),
                          true, i18n("New Plot..."));
  toolBar()->insertButton(Icon("fileopen.xpm"), ID_FILE_OPEN_PLOT,
                          SIGNAL(clicked()), this, SLOT(slotFileOpenPlot()),
                          true, i18n("Open Plot File..."));
  toolBar()->insertButton(Icon("fileopen.xpm"), ID_FILE_OPEN_DATA,
                          SIGNAL(clicked()), this, SLOT(slotFileOpenData()),
                          true, i18n("Open Data File..."));
  toolBar()->insertSeparator();
  toolBar()->insertButton(Icon("filefloppy.xpm"), ID_FILE_SAVE,
                          SIGNAL(clicked()), this, SLOT(slotFileSave()),
                          true, i18n("Save"));
  toolBar()->insertSeparator();
  toolBar()->insertButton(Icon("fileprint.xpm"), ID_FILE_PRINT,
                          SIGNAL(clicked()), this, SLOT(slotFilePrint()),
                          true, i18n("Print..."));
  toolBar()->insertSeparator();
  toolBar()->insertButton(Icon("viewmag+.xpm"), ID_VIEW_ZOOM_IN,
                          SIGNAL(clicked()), this, SLOT(slotViewZoomIn()),
                          true, i18n("Zoom In"));
  toolBar()->insertButton(Icon("viewmag-.xpm"), ID_VIEW_ZOOM_OUT,
                          SIGNAL(clicked()), this, SLOT(slotViewZoomOut()),
                          true, i18n("Zoom Out"));
  toolBar()->insertSeparator();
  toolBar()->insertButton(Icon("reload.xpm"), ID_VIEW_RELOAD,
                          SIGNAL(clicked()), this, SLOT(slotViewReload()),
                          true, i18n("Reload"));
  toolBar()->insertSeparator();
  toolBar()->insertButton(Icon("help.xpm"), ID_HELP,
                          SIGNAL(clicked()), kapp, SLOT(appHelpActivated()),
                          true, i18n("Help"));
#endif
  connect(toolBar(), SIGNAL(highlighted(int, bool)), SLOT(statusCallback(int)));
}

void KplApp::calcStatusBar()
{
  xPrec = QMIN(QMAX(2, int9(log10(QMAX(fabs(view->xmin), fabs(view->xmax)) / view->fxn
                             + 1.0e-37) + 0.999999) +
               int9(log10(view->xscal * view->fxn + 1.0e-37) + 0.999999)), 16);
  yPrec = QMIN(QMAX(2, int9(log10(QMAX(fabs(view->ymin), fabs(view->ymax)) / view->fyn
                             + 1.0e-37) + 0.999999) +
               int9(log10(-view->yscal * view->fyn + 1.0e-37) + 0.999999)), 16);
}

void KplApp::initStatusBar()
{
  QString s1, s2;

  s1 = "  x = " + s2.setNum(-10.0E-33 / 3.0, 'g', xPrec) +",  y = ";
  s1 += s2.setNum(-10.0E-33 / 3.0, 'g', yPrec);
  statusBar()->setInsertOrder(KStatusBar::RightToLeft);
  statusBar()->insertItem(s1, ID_STATUS_VAL);
  statusBar()->insertItem(" 10.00 cm,  15.00 cm,", ID_STATUS_POS);
  statusBar()->insertItem(i18n("Ready."), ID_STATUS_MSG);
  statusBar()->changeItem("", ID_STATUS_VAL);
  statusBar()->changeItem("", ID_STATUS_POS);
}

void KplApp::newStatusBar()
{
  int xold = xPrec;
  int yold = yPrec;
  calcStatusBar();
  if ((xPrec != xold) || (yPrec != yold)) {
    delete statusBar();
    setStatusBar(new KStatusBar(this));
    initStatusBar();
    enableStatusBar(optionsMenu->isItemChecked(ID_OPTIONS_STATUSBAR) ?
                    KStatusBar::Show : KStatusBar::Hide);
  }
}

void KplApp::initKeyAccel()
{
  key_accel = new KAccel(this);
  CONNECT_ACCEL(KAccel::New, slotFileNew(), fileMenu, ID_FILE_NEW);
  CONNECT_ACCEL(KAccel::Open, slotFileOpenPlot(), fileMenu, ID_FILE_OPEN_PLOT);
  CONNECT_ACCEL(KAccel::Save, slotFileSave(), fileMenu, ID_FILE_SAVE);
  CONNECT_ACCEL(KAccel::Close, slotFileClose(), fileMenu, ID_FILE_CLOSE);
  CONNECT_ACCEL(KAccel::Print, slotFilePrint(), fileMenu, ID_FILE_PRINT);
  CONNECT_ACCEL(KAccel::Quit, slotFileQuit(), fileMenu, ID_FILE_QUIT);
  CONNECT_KEY(Key_F5, i18n("Reload"), "Reload", this, slotViewReload());
  CONNECT_KEY(Key_Left, i18n("Scroll Left"), "Scroll Left", view, slotScrollLeft());
  CONNECT_KEY(Key_Right, i18n("Scroll Right"), "Scroll Right", view, slotScrollRight());
  CONNECT_KEY(Key_Up, i18n("Scroll Up"), "Scroll Up", view, slotScrollUp());
  CONNECT_KEY(Key_Down, i18n("Scroll Down"), "Scroll Down", view, slotScrollDown());
  CONNECT_KEY(Key_PageUp, i18n("Scroll Page Up"), "Scroll Page Up", view, slotScrollPageUp());
  CONNECT_KEY(Key_PageDown, i18n("Scroll Page Down"), "Scroll Page Down", view, slotScrollPageDown());
  CONNECT_KEY(CTRL + Key_PageUp, i18n("Scroll Page Left"), "Scroll Page Left", view, slotScrollPageLeft());
  CONNECT_KEY(CTRL + Key_PageDown, i18n("Scroll Page Right"), "Scroll Page Right", view, slotScrollPageRight());
  CONNECT_KEY(CTRL + Key_Home, i18n("Scroll to Left Margin"), "Scroll to Left Margin", view, slotScrollHomeLeft());
  CONNECT_KEY(Key_Home, i18n("Scroll to Bottom Margin"), "Scroll to Bottom Margin", view, slotScrollHomeDown());
  CONNECT_KEY(CTRL + Key_End, i18n("Scroll to Right Margin"), "Scroll to Right Margin", view, slotScrollEndRight());
  CONNECT_KEY(Key_End, i18n("Scroll to Top Margin"), "Scroll to Top Margin", view, slotScrollEndUp());
  key_accel->readSettings();	
  key_accel->changeMenuAccel(viewMenu, ID_VIEW_RELOAD, "Reload");
}

void KplApp::initDocument()
{
  doc = new KplDoc(this);
  doc->newDocument();
}

void KplApp::initView()
{
  view = new KplView(this, fz);
  setView(view);
}

void KplApp::saveOptions(bool saveAll)
{
  QString s;
  QStrList s1, s2, s3;

#if (KDE_VERSION_MAJOR > 1) || (KDE_VERSION_MINOR > 1)
  config->setGroup(KGlobal::instance()->instanceName());
#else
  config->setGroup(kapp->appName());
#endif
	config->writeEntry("RecentFiles", recentFiles);
	if (saveAll) {
    config->writeEntry("ShowToolbar", toolBar()->isVisible());
    config->writeEntry("ShowStatusbar", statusBar()->isVisible());
    config->writeEntry("MenuBarPos", (int) menuBar()->menuBarPos());
    config->writeEntry("ToolBar_Pos", (int) toolBar()->barPos());
    config->writeEntry("ShowItemDialog", itemDlg ? itemDlg->isVisible() : false);
    config->writeEntry("Directory", doc->currentDir());
    config->writeEntry("Width", width());
    config->writeEntry("Height", height());
    config->writeEntry("AutoPlot", optionsMenu->isItemChecked(ID_OPTIONS_AUTOPLOT));
    config->writeEntry("ShowSource", optionsMenu->isItemChecked(ID_OPTIONS_SHOWSOURCE));
    config->writeEntry("BoundingBox", optionsMenu->isItemChecked(ID_OPTIONS_BBOX));
    config->writeEntry("Warning", optionsMenu->isItemChecked(ID_OPTIONS_WARNING));
    config->writeEntry("SaveAtEnd", optionsMenu->isItemChecked(ID_OPTIONS_SAVE_AT_END));
    config->writeEntry("AutoIndexX", doc->aut.ixAuto);
    config->writeEntry("AutoIndexY", doc->aut.iyAuto);
    config->writeEntry("AutoIndexE", doc->aut.ieAuto);
    config->writeEntry("AutoNorm", doc->aut.autoNorm);
    config->writeEntry("AutoErrBar", doc->aut.autoErr);
    config->writeEntry("AutoGrid", doc->aut.autoGrid);
    config->writeEntry("AutoPathHeader", doc->aut.autohPath);
    config->writeEntry("AutoFit", doc->aut.autoFit);
    config->writeEntry("AutoSymbol", doc->aut.autoSymb);
    config->writeEntry("AutoX0", doc->aut.x0Auto);
    config->writeEntry("AutoY0", doc->aut.y0Auto);
    config->writeEntry("AutoXl", doc->aut.xlAuto);
    config->writeEntry("AutoYl", doc->aut.ylAuto);
    config->writeEntry("FitErrorColumn", doc->aut.fitErrCol);
    config->writeEntry("FitNonLinear", doc->aut.fitNonLin);
    config->writeEntry("FitShowDialog", doc->aut.fitShowDlg);
    config->writeEntry("FitSaveParam", doc->aut.fitSavePar);
    config->writeEntry("FitFollow", doc->aut.fitFollow);
    config->writeEntry("FitMaxIterations", doc->aut.fitMaxIt);
    config->writeEntry("FitTolerance", doc->aut.fitTol);
    config->writeEntry("FitSymbol", doc->aut.fitSymb);
    config->writeEntry("FitColor", doc->aut.fitColor);
    config->writeEntry("FitPath", doc->aut.fitPath);
    config->writeEntry("FitName", doc->aut.fitName);
    config->writeEntry("ErrorModelPath", doc->aut.errModPath);
    config->writeEntry("ErrorModelName", doc->aut.errModName);
    for (int i = 0; i < NPMAX; i++) {
      s1.insert(i, s.setNum((int) doc->aut.fitPar[i]));
      s2.insert(i, s.setNum(doc->aut.pFit[i], doc->aut.format, doc->aut.prec));
      s3.insert(i, s.setNum(doc->aut.pErrMod[i], doc->aut.format, doc->aut.prec));
    }
    config->writeEntry("FitEnableParam", s1, ' ');
    config->writeEntry("FitParam", s2, ' ');
    config->writeEntry("ErrorModelParam", s3, ' ');
    config->writeEntry("FormatX", doc->aut.xf);
    config->writeEntry("FormatY", doc->aut.yf);
    config->writeEntry("ColorFrame", doc->aut.colFrame);
    config->writeEntry("ColorGrid", doc->aut.colGrid);
    config->writeEntry("ColorData", doc->aut.colData);
    s.sprintf("%c", doc->aut.format);
    config->writeEntry("OutputFormat", s);
    config->writeEntry("Precision", doc->aut.prec);
    config->writeEntry("Zoom", view->fZoom);
    config->writeEntry("AutoLetX", doc->aut.sxAuto);
    config->writeEntry("AutoLetY", doc->aut.syAuto);
    config->writeEntry("AutoLetH", doc->aut.shAuto);
  }
  config->sync();
  key_accel->writeSettings();	
}

void KplApp::readOptions()
{
  unsigned int i;
  char s1[256];
  QString s, s2;

#if (KDE_VERSION_MAJOR > 1) || (KDE_VERSION_MINOR > 1)
  config->setGroup(KGlobal::instance()->instanceName());
#else
  config->setGroup(kapp->appName());
#endif
  config->setDollarExpansion();
  mainWidth = config->readNumEntry("Width", 640);
  mainHeight = config->readNumEntry("Height", 512);
  resize(mainWidth, mainHeight);
	recentFiles.setAutoDelete(true);
	config->readListEntry("RecentFiles", recentFiles);
	for (i = 0; i < recentFiles.count(); i++)
    recentFilesMenu->insertItem(recentFiles.at(i));
  bool b = config->readBoolEntry("ShowToolbar", true);
  if (!b)
    enableToolBar(KToolBar::Hide);
  optionsMenu->setItemChecked(ID_OPTIONS_TOOLBAR, b);
  b = config->readBoolEntry("ShowStatusbar", true);
  optionsMenu->setItemChecked(ID_OPTIONS_STATUSBAR, b);
  if (!b)
    enableStatusBar(KStatusBar::Hide);
  menuBar()->setMenuBarPos((KMenuBar::menuPosition) config->readNumEntry("MenuBarPos",
                                                                         KMenuBar::Top));
  toolBar()->setBarPos((KToolBar::BarPosition) config->readNumEntry("ToolBar_Pos",
                                                                    KToolBar::Top));
  if (config->readBoolEntry("ShowItemDialog", false))
    slotEditItems();
#if (KDE_VERSION_MAJOR > 1) || (KDE_VERSION_MINOR > 1)
  QString kplDir = KGlobal::dirs()->findResourceDir("data",
                                                    KGlobal::instance()->instanceName() +
						    "/demo1.plo") +
                   QString(KGlobal::instance()->instanceName());
#else
  QString kplDir = kapp->kde_datadir() + "/" + kapp->appName();
#endif
  doc->setCurrentDir(config->readEntry("Directory", kplDir));
  doc->aut.showSource = config->readBoolEntry("ShowSource", false);
  optionsMenu->setItemChecked(ID_OPTIONS_SHOWSOURCE, doc->aut.showSource);
  optionsMenu->setItemChecked(ID_OPTIONS_AUTOPLOT, config->readBoolEntry("AutoPlot", true));
  optionsMenu->setItemChecked(ID_OPTIONS_BBOX, config->readBoolEntry("BoundingBox"));
  optionsMenu->setItemChecked(ID_OPTIONS_WARNING, config->readBoolEntry("Warning", true));
  optionsMenu->setItemChecked(ID_OPTIONS_SAVE_AT_END, config->readBoolEntry("SaveAtEnd"));
  doc->aut.ixAuto = config->readNumEntry("AutoIndexX");
  doc->aut.iyAuto = config->readNumEntry("AutoIndexY", 1);
  doc->aut.ieAuto = config->readNumEntry("AutoIndexE", 2);
  doc->aut.autoNorm = config->readBoolEntry("AutoNorm", true);
  doc->aut.autoErr = config->readBoolEntry("AutoErrBar");
  doc->aut.autoGrid = config->readNumEntry("AutoGrid", 2);
  doc->aut.autohPath = config->readBoolEntry("AutoPathHeader", true);
  doc->aut.autoFit = config->readBoolEntry("AutoFit");
  doc->aut.autoSymb = config->readNumEntry("AutoSymbol", 1);
  doc->aut.x0Auto = config->readDoubleNumEntry("AutoX0", 4.0);
  doc->aut.y0Auto = config->readDoubleNumEntry("AutoY0", 3.0);
  doc->aut.xlAuto = config->readDoubleNumEntry("AutoXl", 15.0);
  doc->aut.ylAuto = config->readDoubleNumEntry("AutoYl", 10.0);
  doc->aut.fitErrCol = config->readBoolEntry("FitErrorColumn");
  doc->aut.fitNonLin = config->readBoolEntry("FitNonLinear", true);
  doc->aut.fitShowDlg = config->readBoolEntry("FitShowDialog");
  doc->aut.fitSavePar = config->readBoolEntry("FitSaveParam");
  doc->aut.fitFollow = config->readBoolEntry("FitFollow");
  doc->aut.fitMaxIt = config->readNumEntry("FitMaxIterations", 100);
  doc->aut.fitTol = config->readDoubleNumEntry("FitTolerance", 1e-14);
  doc->aut.fitSymb = config->readNumEntry("FitSymbol", 1);
  doc->aut.fitColor = config->readLongNumEntry("FitColor", black.rgb());
  strcpy(doc->aut.fitPath, config->readEntry("FitPath", kplDir + "/fkt.so"));
  strcpy(doc->aut.fitName, config->readEntry("FitName", "polynom"));
  strcpy(doc->aut.errModPath, config->readEntry("ErrorModelPath", kplDir + "/fkt.so"));
  strcpy(doc->aut.errModName, config->readEntry("ErrorModelName", "polynom"));
  strcpy(s1, config->readEntry("FitEnableParam", "0"));
  doc->aut.fitPar[0] = ((s2 = strtok(s1, " "))) ? s2.toInt() : false;
  for (i = 1; i < NPMAX; i++)
    doc->aut.fitPar[i] = ((s2 = strtok(0, " "))) ? s2.toInt() : false;
  strcpy(s1, config->readEntry("FitParam", "0"));
  doc->aut.pFit[0] = ((s2 = strtok(s1, " "))) ? s2.toDouble() : 0.0;
  for (i = 1; i < NPMAX; i++)
    doc->aut.pFit[i] = ((s2 = strtok(0, " "))) ? s2.toDouble() : 0.0;
  strcpy(s1, config->readEntry("ErrorModelParam", "1.e-14 0.001"));
  doc->aut.pErrMod[0] = ((s2 = strtok(s1, " "))) ? s2.toDouble() : 0.0;
  for (i = 1; i < NPMAX; i++)
    doc->aut.pErrMod[i] = ((s2 = strtok(0, " "))) ? s2.toDouble() : 0.0;
  doc->aut.xf = config->readDoubleNumEntry("FormatX", 21.0);
  doc->aut.yf = config->readDoubleNumEntry("FormatY", 29.7);
  doc->aut.colFrame = config->readLongNumEntry("ColorFrame");
  doc->aut.colGrid = config->readLongNumEntry("ColorGrid");
  doc->aut.colData = config->readLongNumEntry("ColorData");
  doc->aut.format = *((const char *) config->readEntry("OutputFormat", "g"));
  doc->aut.prec = config->readNumEntry("Precision", 6);
  fz = config->readDoubleNumEntry("Zoom", 1.0);
  config->setDollarExpansion(false);
  strcpy(doc->aut.sxAuto, config->readEntry("AutoLetX", "x"));
  strcpy(doc->aut.syAuto, config->readEntry("AutoLetY", "y"));
  strcpy(doc->aut.shAuto, config->readEntry("AutoLetH", ""));
 }

void KplApp::enableCommand(int id, bool enable)
{
  menuBar()->setItemEnabled(id, enable);
  toolBar()->setItemEnabled(id, enable);
}

void KplApp::toggleOption(int id)
{
  optionsMenu->setItemChecked(id, !optionsMenu->isItemChecked(id));
  slotStatusMsg(i18n("Ready."));
}

void KplApp::openRead(fileTypes type)
{
  if (optionsMenu->isItemChecked(ID_OPTIONS_WARNING))
    if (!queryClose())
      return;
  QString u = KFileDialog::getOpenFileURL(doc->currentDir(), type == DATAFILE ?
                                          "*.dat *.DAT" : "*.plo *.PLO");
  if (!u.isEmpty()) {
    doc->openDocument(u, type);
    if (itemDlg)
      itemDlg->updItemList();
  }
}

bool KplApp::save(bool current)
{
  bool success = false;
  KURL *u = 0;
  QString s;

  if ((doc->currentFile() != i18n("Untitled")) && current &&
      (doc->fType == PLOTFILE)) {
    u = new KURL(doc->currentFile());
    if (u->isLocalFile())
      success = doc->saveDocument(doc->currentFile());
    else
      success = doc->saveDocument(QDir::homeDirPath() + "/kpltmp.tmp");
  } else {
    if (getWriteURL(s, "*.plo"))
      success = doc->saveDocument(s);
    if (success) {
      u = new KURL(s);
      if (u->isLocalFile()) {
        QFileInfo fi(s);
        doc->setCurrentDir(fi.dirPath(true));
        doc->setCurrentFile(fi.absFilePath());
      } else {
#if (KDE_VERSION_MAJOR > 1) || (KDE_VERSION_MINOR > 1)
        doc->setCurrentDir(u->protocol() + ":" + u->host() + u->directory());
#else
        doc->setCurrentDir(u->directoryURL());
#endif
        doc->setCurrentFile(u->url());
      }
      doc->fType = PLOTFILE;
    }
  }
  if (success) {
    if (!u->isLocalFile()) {
#if (KDE_VERSION_MAJOR > 1) || (KDE_VERSION_MINOR > 1)
      doc->kIO = new KIOJob;
#else
      doc->kIO = new KFM;
#endif
      doc->connectIOfinished();
      doc->kIO->copy(QDir::homeDirPath() + "/kpltmp.tmp",
                     doc->currentFile());
    }
    updWin();
  }
  if (u)
    delete u;
  return success;
}

bool KplApp::getWriteURL(QString &url, const char *filter)
{
  url = KFileDialog::getSaveFileURL(doc->currentDir(), filter);
  if (url.isEmpty())
    return false;
  KURL u = KURL(url);
  if (u.isLocalFile()) {
    url = u.path();
    QFileInfo fi(url);
    if (fi.exists())
      if (fi.isWritable())
#if (KDE_VERSION_MAJOR > 1) || (KDE_VERSION_MINOR > 1)
        return (KMessageBox::warningYesNo(this, url + "\n" +
                                          i18n("A file with this name exists already.\n"
                                         "Do you want to overwrite it?")) == KMessageBox::Yes);
      else {
        KMessageBox::sorry(this, i18n("Writing to this file not possible."));
#else
        return (KMsgBox::yesNo(this, i18n("Warning"),
                               i18n("A file with this name exists already.\n"
                                    "Do you want to overwrite it?"),
                               KMsgBox::EXCLAMATION) == 1);
      else {
        KMsgBox::message(this, i18n("Sorry"),
                         i18n("Writing to this file not possible."),
                         KMsgBox::EXCLAMATION);
#endif
        return false;
      }
  }
  return true;
}

bool KplApp::queryClose()
{
  if (doc->isModified())
    do {
#if (KDE_VERSION_MAJOR > 1) || (KDE_VERSION_MINOR > 1)
      int i = KMessageBox::warningYesNoCancel(this, i18n("The current plot has been changed.\n"
                                                     "Save file?"));
      if (i == KMessageBox::Cancel)
        return false;
      if (i == KMessageBox::No)
#else
      int i = KMsgBox::yesNoCancel(this, i18n("Warning"),
                                   i18n("The current plot has been changed.\n"
                                        "Save file?"),
                                   KMsgBox::EXCLAMATION);
      if (i == 3)
        return false;
      if (i == 2)
#endif
        break;
    } while (!save(true));
  return true;
}

void KplApp::addRecentFile(const char* path)
{
  if (recentFiles.find(path) == -1) {
    if (recentFiles.count() < 5)
      recentFiles.insert(0, path);
    else {
      recentFiles.remove(4);
      recentFiles.insert(0, path);
    }
    recentFilesMenu->clear();
    for (int i = 0; i < (int) recentFiles.count(); i++)
      recentFilesMenu->insertItem(recentFiles.at(i));
	}
}

bool KplApp::queryExit()
{
  saveOptions(optionsMenu->isItemChecked(ID_OPTIONS_SAVE_AT_END));
  return true;
}

void KplApp::saveProperties(KConfig *_cfg)
{
  if (doc->currentFile() == i18n("Untitled") && !doc->isModified())
    return;
  QString filename = doc->currentFile();
  if (filename == i18n("Untitled"))
    filename = QDir::homeDirPath() + "/" + i18n("Untitled");
  _cfg->writeEntry("filename", filename);
  _cfg->writeEntry("modified", doc->isModified());
  if (doc->isModified())
    doc->saveDocument(kapp->tempSaveName(filename));
}

void KplApp::readProperties(KConfig *_cfg)
{
  QString filename = _cfg->readEntry("filename", "");
  bool modified = _cfg->readBoolEntry("modified", false);
  if (modified) {
    bool b_canRecover;
    QString tempname = kapp->checkRecoverFile(filename, b_canRecover);
    if (b_canRecover) {
      doc->openDocument(tempname, PLOTFILE);
      QFile::remove(tempname);
      doc->setModified();
      if (filename == (QDir::homeDirPath() + "/" + i18n("Untitled")))
        doc->setCurrentFile(i18n("Untitled"));
      else {
        KURL u = KURL(filename);
        if (u.isLocalFile()) {
          QFileInfo fi(filename);
          doc->setCurrentFile(fi.absFilePath());
          doc->setCurrentDir(fi.dirPath(true));
        } else {
#if (KDE_VERSION_MAJOR > 1) || (KDE_VERSION_MINOR > 1)
          doc->setCurrentDir(u.protocol() + ":" + u.host() + u.directory());
#else
          doc->setCurrentDir(u.directoryURL());
#endif
          doc->setCurrentFile(u.url());
        }
      }
    }
  } else
    if (!filename.isEmpty()) {
      doc->openDocument(filename);
      KURL u = KURL(filename);
      if (u.isLocalFile()) {
        QFileInfo fi(filename);
        doc->setCurrentFile(fi.absFilePath());
        doc->setCurrentDir(fi.dirPath(true));
      } else {
#if (KDE_VERSION_MAJOR > 1) || (KDE_VERSION_MINOR > 1)
        doc->setCurrentDir(u.protocol() + ":" + u.host() + u.directory());
#else
        doc->setCurrentDir(u.directoryURL());
#endif
        doc->setCurrentFile(u.url());
      }
    }
  updWin();
}		

void KplApp::statusCallback(int id) {
  switch (id) {
    ON_STATUS_MSG(ID_FILE_NEW_WINDOW, i18n("Opens a new application window"))
    ON_STATUS_MSG(ID_FILE_NEW, i18n("Creates a new plot"))
    ON_STATUS_MSG(ID_FILE_OPEN_PLOT, i18n("Opens an existing plot file"))
    ON_STATUS_MSG(ID_FILE_OPEN_DATA, i18n("Opens an existing data file"))
    ON_STATUS_MSG(ID_FILE_SAVE, i18n("Saves the current plot"))
    ON_STATUS_MSG(ID_FILE_SAVE_AS, i18n("Saves the plot under a "
                                        "different name"))
    ON_STATUS_MSG(ID_FILE_CLOSE, i18n("Closes the current plot"))
    ON_STATUS_MSG(ID_FILE_PLOT, i18n("Displays the current plot"))
    ON_STATUS_MSG(ID_FILE_PS, i18n("Prints to PostScript file"))
    ON_STATUS_MSG(ID_FILE_PRINT, i18n("Prints the current plot"))
    ON_STATUS_MSG(ID_FILE_CLOSE_WINDOW, i18n("Closes the current window"))
    ON_STATUS_MSG(ID_FILE_QUIT, i18n("Exits the program"))

    ON_STATUS_MSG(ID_EDIT, i18n("Edits plot items"))

    ON_STATUS_MSG(ID_VIEW_ZOOM_IN, i18n("Increases magnification by 20 %"))
    ON_STATUS_MSG(ID_VIEW_ZOOM_OUT, i18n("Decreases magnification by 20 %"))
    ON_STATUS_MSG(ID_VIEW_ZOOM, i18n("Sets magnification"))
    ON_STATUS_MSG(ID_VIEW_RELOAD, i18n("Reloads file"))
    ON_STATUS_MSG(ID_VIEW_AUTO_RELOAD, i18n("Reloads file periodically"))

    ON_STATUS_MSG(ID_OPTIONS_TOOLBAR, i18n("Enables / disables the toolbar"))
    ON_STATUS_MSG(ID_OPTIONS_STATUSBAR, i18n("Enables / disables "
                                             "the statusbar"))
    ON_STATUS_MSG(ID_OPTIONS_AUTOPLOT, i18n("Enables / disables "
                                            "automatic plotting"))
    ON_STATUS_MSG(ID_OPTIONS_SHOWSOURCE, i18n("Enables / disables "
                                              "display of function source"))
    ON_STATUS_MSG(ID_OPTIONS_BBOX, i18n("Enables / disables "
                                        "calculation of PS bounding box"))
    ON_STATUS_MSG(ID_OPTIONS_WARNING, i18n("Enables / disables warning "
                                           "on unsaved changes"))
    ON_STATUS_MSG(ID_OPTIONS_SAVE_AT_END, i18n("Enables / disables saving "
                                               "options at end"))
    ON_STATUS_MSG(ID_OPTIONS_KEYS, i18n("Displays key bindings dialog"))
    ON_STATUS_MSG(ID_OPTIONS_BASIC, i18n("Displays basic settings dialog"))
    ON_STATUS_MSG(ID_OPTIONS_AUTO, i18n("Displays autoplot settings dialog"))
    ON_STATUS_MSG(ID_OPTIONS_AUTOFIT, i18n("Displays autofit settings dialog"))
    ON_STATUS_MSG(ID_OPTIONS_SAVE, i18n("Saves options"))
    ON_STATUS_MSG(ID_HELP, i18n("Help"))
  }
}

void KplApp::slotFileNew()
{
  if (queryClose()) {
    doc->newDocument();
    if (itemDlg)
      itemDlg->updItemList();
    updWin();
    view->eraseWidget();
    slotEditItems();
  }
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotFileOpenPlot()
{
  statusBar()->message(i18n("Opening file..."));
  openRead(PLOTFILE);
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotFileOpenData()
{
  statusBar()->message(i18n("Opening file..."));
  openRead(DATAFILE);
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotFileOpenRecent(int id)
{
  statusBar()->message(i18n("Opening file..."));
  if (optionsMenu->isItemChecked(ID_OPTIONS_WARNING))
    if (!queryClose()) {
      slotStatusMsg(i18n("Ready."));
      return;
    }
#if (KDE_VERSION_MAJOR > 1) || (KDE_VERSION_MINOR > 1)
  openDocumentFile(recentFiles.at(recentFilesMenu->indexOf(id)));
#else
  openDocumentFile(recentFiles.at(id));
#endif
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotFileSave()
{
  statusBar()->message(i18n("Saving file..."));
  save(true);
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotFileSaveAs()
{
  statusBar()->message(i18n("Saving file under different name..."));
  save(false);
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotFileClose()
{
  statusBar()->message(i18n("Closing file..."));
  if (queryClose()) {
    doc->newDocument();
    if (itemDlg)
      itemDlg->updItemList();
    updWin();
    view->eraseWidget();
  }
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotFilePlot()
{
  statusBar()->message(i18n("Plotting..."));
  plot = doc->itd.count();
  slotPlotCond();
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotPlotCond()
{
  if (plot)
    view->paintWidget();
  newStatusBar();
}

void KplApp::slotFilePS()
{
  QString s;
  FILE *f = 0;

  statusBar()->message(i18n("Generating PostScript file..."));
  if (getWriteURL(s, "*.ps *.eps"))
    if ((f = fopen(s, "w"))) {
      view->page->setCursor(waitCursor);
      if (doc->itd.count()) {
        PSGraph g;
        g.psInit(f, doc->aut.xf, doc->aut.yf);
        for (KplItem *item = doc->itd.first(); item; item = doc->itd.next())
          item->draw(&g);
        g.psExit();
      }
      if (optionsMenu->isItemChecked(ID_OPTIONS_BBOX)) {
        QString s2;
        KProcess kp;
        kp << "gs" << "-dNOPAUSE" << "-dBATCH" << "-q" << "-sDEVICE=pbmraw"
           << ("-sOutputFile=" + QDir::homeDirPath() + "/kpltmp.pbm") << "-r72"
           << s2.sprintf("-g%ix%i", qRound(28.34645669 * doc->aut.xf),
                         qRound(28.34645669 * doc->aut.yf))
           << s;
        kp.start(KProcess::Block);
        QImage image;
        if (image.load(QDir::homeDirPath() + "/kpltmp.pbm", "PBMRAW")) {
          int x, y, xl = 0, yo = 0;
          int xr = image.width();
          int yu = image.height();
          for (y = 0; y < image.height(); y++)
            for (x = 0; x < image.width(); x++)
              if (!(image.pixel(x, y) & 0xffffff)) {
                yo = y;
                xl = xr = x;
                goto yo_found;
              }
  yo_found:
          for (y = image.height() - 1; y > yo; y--)
            for (x = 0; x < image.width(); x++)
              if (!(image.pixel(x, y) & 0xffffff)) {
                yu = y;
                xl = QMIN(xl, x);
                xr = QMAX(xr, x);
                goto yu_found;
              }
  yu_found:
          for (y = yo; y <= yu; y++){
            for (x = 0; x < xl; x++)
              if (!(image.pixel(x, y) & 0xffffff)) {
                xl = x;
                break;
              }
            for (x = image.width() - 1; x > xr; x--)
              if (!(image.pixel(x, y) & 0xffffff)) {
                xr = x;
                break;
              }
          }
          xl = QMAX(xl - 2, 0);
          xr = QMIN(xr + 2, image.width() - 1);
          yu = QMAX(image.height() - yu - 2, 0);
          yo = QMIN(image.height() - yo + 2, image.height() - 1);
          remove(QDir::homeDirPath() + "/kpltmp.pbm");
          f = fopen(s, "r+");
          fprintf(f, "%%!PS-Adobe-2.0 EPSF-1.2\n%%%%BoundingBox: %i %i %i %i      ",
                  xl, yu, xr, yo);
          fclose(f);
        } else
#if (KDE_VERSION_MAJOR > 1) || (KDE_VERSION_MINOR > 1)
          KMessageBox::error(this, i18n("Bitmap file for bounding box calculation not loaded!"));
#else
          KMsgBox::message(this, i18n("Error"),
                           i18n("Bitmap file for bounding box calculation not loaded!"),
                           KMsgBox::EXCLAMATION);
#endif
      }
      QFileInfo fi(s);
      doc->setCurrentDir(fi.dirPath(true));
      view->page->setCursor(crossCursor);
    }
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotFilePrint()
{
  statusBar()->message(i18n("Printing..."));
  QPrinter printer;
  if (printer.setup(this))
    view->print(&printer);
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotFileNewWindow()
{
  (new KplApp)->show();
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotFileCloseWindow()
{
  close();
}

void KplApp::slotFileQuit()
{
  slotStatusMsg(i18n("Exiting..."));
  if (memberList)
    for (KTMainWindow *w = memberList->first(); w; w = memberList->first())
      if (!w->close())
        break;
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotEditItems()
{
  if (!itemDlg) {
    itemDlg = new ItemDlg(doc);
    connect(itemDlg, SIGNAL(displayMessage(const char *)),
            SLOT(slotStatusMsg(const char *)));
  }
  itemDlg->show();
  itemDlg->raise();
}

void KplApp::slotViewZoomIn()
{
  view->fZoom = 0.01 * (QMIN(qRound(100 * view->fZoom) + 20, 2000));
  shrink();
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotViewZoomOut()
{
  view->fZoom = 0.01 * (QMAX(qRound(100 * view->fZoom) - 20, 1));
  shrink();
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotViewZoom()
{
  statusBar()->message(i18n("Setting magnification..."));
  ZoomDlg dlg(this);
  dlg.factor->setValue(qRound(100 * view->fZoom));
  if (dlg.exec() == dlg.Accepted) {
    view->fZoom = 0.01 * dlg.factor->value();
    shrink();
  }
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotViewReload()
{
  statusBar()->message(i18n("Reloading file..."));
  menuBar()->setItemChecked(ID_OPTIONS_AUTOPLOT, true);
  doc->openDocument(doc->currentFile(), doc->fType);
  if (itemDlg)
    itemDlg->updItemList();
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotViewAutoReload()
{
  statusBar()->message(i18n("Reloading file periodically..."));
  RefreshDlg dlg(this);
  bool timerActive = timer->isActive();
  dlg.setWidgets(tRefresh, timerActive);
  if (dlg.exec() == dlg.Accepted) {
    int tRef;
    bool act;
    dlg.getValues(&tRef, &act);
    if (timerActive && ((tRef != tRefresh) || (act != timerActive)))
      timer->stop();
    if (act && ((tRef != tRefresh) || (act != timerActive))) {
      timer->start(1000 * tRef);
    }
    tRefresh = tRef;
  }
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotOptionsToolBar()
{
  toggleOption(ID_OPTIONS_TOOLBAR);
  enableToolBar();
}

void KplApp::slotOptionsStatusBar()
{
  toggleOption(ID_OPTIONS_STATUSBAR);
  enableStatusBar();
}

void KplApp::slotOptionsWarning()
{
  toggleOption(ID_OPTIONS_WARNING);
}

void KplApp::slotOptionsAutoPlot()
{
  toggleOption(ID_OPTIONS_AUTOPLOT);
  if (optionsMenu->isItemChecked(ID_OPTIONS_AUTOPLOT))
    slotFilePlot();
}

void KplApp::slotOptionsShowSource()
{
  toggleOption(ID_OPTIONS_SHOWSOURCE);
  doc->aut.showSource = optionsMenu->isItemChecked(ID_OPTIONS_SHOWSOURCE);
}

void KplApp::slotOptionsBBox()
{
  toggleOption(ID_OPTIONS_BBOX);
}

void KplApp::slotOptionsSaveAtEnd()
{
  toggleOption(ID_OPTIONS_SAVE_AT_END);
}

void KplApp::slotOptionsKeys()
{
  statusBar()->message(i18n("Key bindings..."));
  KKeyDialog::configureKeys(key_accel, false);
  key_accel->changeMenuAccel(viewMenu, ID_VIEW_RELOAD, "Reload");
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotOptionsBasic()
{
  statusBar()->message(i18n("Basic settings..."));
  BasicDlg dlg(this, &doc->aut);
  if (dlg.exec() == dlg.Accepted)
    shrink();
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotOptionsAuto()
{
  statusBar()->message(i18n("Autoplot settings..."));
  AutoDlg dlg(this, doc);
  dlg.exec();
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotOptionsAutoFit()
{
  statusBar()->message(i18n("Autofit settings..."));
  AutoFitDlg dlg(this, doc);
  dlg.exec();
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotOptionsSave()
{
  saveOptions(true);
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotStatusMsg(const char *text)
{
  statusBar()->clear();
  statusBar()->changeItem(text, ID_STATUS_MSG);
}

void KplApp::slotStatusHelpMsg(const char *text)
{
  statusBar()->message(text, 3000);
}

void KplApp::slotOpenList(QStrList *list)
{
  for (char *u = list->first(); u; u = list->next())
    if (u == list->getFirst()) {
      if (optionsMenu->isItemChecked(ID_OPTIONS_WARNING) ? queryClose() : true)
        openDocumentFile(u);
    } else {
      KplApp* kpl = new KplApp;
      if (kpl->openDocumentFile(u))
        kpl->show();
      else
        delete kpl;
    }
}

#if (KDE_VERSION_MAJOR < 2) && (KDE_VERSION_MINOR < 2)
void KplApp::slotDropEvent(KDNDDropZone *zone)
{
  QStrList list = zone->getURLList();
  slotOpenList(&list);
}
#endif

void KplApp::slotUpdate()
{
  updWin();
  if (doc->itd.count())
    slotPlotCond();
  else
    view->eraseWidget();
}

void KplApp::updWin()
{
  if (doc->currentFile() != i18n("Untitled")) {
    setCaption(doc->currentFile() + " - kpl");
    addRecentFile(doc->currentFile());
  } else
    setCaption("kpl");
  enableCommand(ID_FILE_OPEN_RECENT, recentFiles.count());
  bool nItems = doc->itd.count();
  enableCommand(ID_FILE_SAVE_AS, nItems);
  enableCommand(ID_FILE_CLOSE, nItems);
  enableCommand(ID_FILE_PLOT, nItems);
  enableCommand(ID_FILE_PS, nItems);
  enableCommand(ID_FILE_PRINT, nItems);
  enableCommand(ID_FILE_SAVE, doc->isModified());
  enableCommand(ID_VIEW_RELOAD, (doc->currentFile() != i18n("Untitled")));
  plot = optionsMenu->isItemChecked(ID_OPTIONS_AUTOPLOT) && nItems;
}

void KplApp::updStatus(int x, int y)
{
  double xcm, ycm, xr, yr;
  QString s1, s2;

  if (statusBar()->isVisible()) {
    if ((!view->i2cm(x, y, &xcm, &ycm)) && plot) {
      view->i2r(x, y, view->fxn, view->fyn, &xr, &yr);
      s1 = "  x = " + s2.setNum(xr, 'g', xPrec) +",  y = ";
      s1 += s2.setNum(yr, 'g', yPrec);
      statusBar()->changeItem(s1, ID_STATUS_VAL);
      statusBar()->changeItem(s1.sprintf("%6.2f cm, %6.2f cm,", xcm, ycm),
                              ID_STATUS_POS);
    } else {
      statusBar()->changeItem("", ID_STATUS_VAL);
      statusBar()->changeItem(s1.sprintf("%6.2f cm, %6.2f cm", xcm, ycm),
                              ID_STATUS_POS);
    }
  }
}

void KplApp::clearStatus()
{
  if (statusBar()->isVisible()) {
    statusBar()->changeItem("", ID_STATUS_VAL);
    statusBar()->changeItem("", ID_STATUS_POS);
  }
}

void KplApp::shrink()
{
  view->layout();
  if ((QMAX(view->xPict, 120) < view->scrx) || (view->yPict < view->scry)) {
    int no = 1 + (toolBar()->sizeHint().width() - 1) / width();
    int w = QMIN(QMAX(view->xPict, 120), view->scrx) - view->scrx + width() + 2;
    int n = 1 + (toolBar()->sizeHint().width() - 1) / w;
    resize(w, QMIN(view->yPict, view->scry) - view->scry + height() +
           menuBar()->heightForWidth(w) - menuBar()->height() +
           (n - no) * (toolBar()->height() / no) + 2);
    newStatusBar();
  } else
    slotPlotCond();
}

void KplApp::showPopupMenu()
{
  popupMenu->clear();
#if (KDE_VERSION_MAJOR > 1) || (KDE_VERSION_MINOR > 1)
  popupMenu->insertItem(BarIcon("filenew"), i18n("&New Plot..."), this,
                        SLOT(slotFileNew()), 0, ID_FILE_NEW);
  popupMenu->insertItem(BarIcon("fileopen"), i18n("&Open Plot File..."), this,
                        SLOT(slotFileOpenPlot()), 0, ID_FILE_OPEN_PLOT);
  popupMenu->insertItem(BarIcon("fileopen"), i18n("Open &Data File..."), this,
                        SLOT(slotFileOpenData()), 0, ID_FILE_OPEN_DATA);
  if (doc->isModified())
    popupMenu->insertItem(BarIcon("filefloppy") ,i18n("&Save"), this,
                          SLOT(slotFileSave()), 0, ID_FILE_SAVE);
  if (doc->itd.count()) {
    popupMenu->insertItem(i18n("Save &As..."), this, SLOT(slotFileSaveAs()),
                          0, ID_FILE_SAVE_AS);
    popupMenu->insertItem(i18n("PostScript O&utput..."), this, SLOT(slotFilePS()),
                          0, ID_FILE_PS);
    popupMenu->insertItem(BarIcon("fileprint"), i18n("&Print..."), this,
                          SLOT(slotFilePrint()), 0, ID_FILE_PRINT);
  }
  popupMenu->insertSeparator();
  popupMenu->insertItem(i18n("&Edit"), this, SLOT(slotEditItems()), 0, ID_EDIT);
  if (doc->itd.count()) {
    popupMenu->insertSeparator();
    popupMenu->insertItem(BarIcon("viewmag+"), i18n("Zoom &In"), this,
                          SLOT(slotViewZoomIn()), 0, ID_VIEW_ZOOM_IN);
    popupMenu->insertItem(BarIcon("viewmag-"), i18n("Zoom &Out"), this,
                          SLOT(slotViewZoomOut()), 0, ID_VIEW_ZOOM_OUT);
  }
  if (doc->currentFile() != i18n("Untitled"))
    popupMenu->insertItem(BarIcon("reload"), i18n("&Reload"), this,
                          SLOT(slotViewReload()), 0, ID_VIEW_RELOAD);
#else
  popupMenu->insertItem(Icon("filenew.xpm"), i18n("&New Plot..."), this,
                        SLOT(slotFileNew()), 0, ID_FILE_NEW);
  popupMenu->insertItem(Icon("fileopen.xpm"), i18n("&Open Plot File..."), this,
                        SLOT(slotFileOpenPlot()), 0, ID_FILE_OPEN_PLOT);
  popupMenu->insertItem(Icon("fileopen.xpm"), i18n("Open &Data File..."), this,
                        SLOT(slotFileOpenData()), 0, ID_FILE_OPEN_DATA);
  if (doc->isModified())
    popupMenu->insertItem(Icon("filefloppy.xpm") ,i18n("&Save"), this,
                          SLOT(slotFileSave()), 0, ID_FILE_SAVE);
  if (doc->itd.count()) {
    popupMenu->insertItem(i18n("Save &As..."), this, SLOT(slotFileSaveAs()),
                          0, ID_FILE_SAVE_AS);
    popupMenu->insertItem(i18n("PostScript O&utput..."), this, SLOT(slotFilePS()),
                          0, ID_FILE_PS);
    popupMenu->insertItem(Icon("fileprint.xpm"), i18n("&Print..."), this,
                          SLOT(slotFilePrint()), 0, ID_FILE_PRINT);
  }
  popupMenu->insertSeparator();
  popupMenu->insertItem(i18n("&Edit"), this, SLOT(slotEditItems()), 0, ID_EDIT);
  if (doc->itd.count()) {
    popupMenu->insertSeparator();
    popupMenu->insertItem(Icon("viewmag+.xpm"), i18n("Zoom &In"), this,
                          SLOT(slotViewZoomIn()), 0, ID_VIEW_ZOOM_IN);
    popupMenu->insertItem(Icon("viewmag-.xpm"), i18n("Zoom &Out"), this,
                          SLOT(slotViewZoomOut()), 0, ID_VIEW_ZOOM_OUT);
  }
  if (doc->currentFile() != i18n("Untitled"))
    popupMenu->insertItem(Icon("reload.xpm"), i18n("&Reload"), this,
                          SLOT(slotViewReload()), 0, ID_VIEW_RELOAD);
#endif
  popupMenu->exec(QCursor::pos());
}
