/***************************************************************************
                          kplitem.cpp  -  description
                             -------------------                                         

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Wed Aug 25 1999
    copyright            : (C) 2000 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include <ksimpleconfig.h>
#include "kplitem.h"
#include "kplgraph.h"

KplItem::KplItem()
{
  active = true;
}

KplItem::KplItem(KSimpleConfig *plo)
{
  active = plo->readBoolEntry("active", true);
}

KplItem::~KplItem()
{
}

void KplItem::freeItem(KplDoc *)
{
}

void KplItem::writePlo(FILE *f)
{
  if (!active)
    fprintf(f, "active=false\n");
}

void KplItem::normalize(double, double)
{
}

void KplItem::expoItem(int *, int *, double *, double *)
{
}

void KplItem::minMax(double *, double *, double *, double *)
{
}

ScaledItem::ScaledItem()
{
  symb = 0;
  color = 0;
  fx = fy = 1.0;
}

ScaledItem::ScaledItem(AutoStruct *aut)
{
  symb = aut->autoSymb;
  color = aut->colData;
  fx = fy = 1.0;
}

ScaledItem::ScaledItem(KSimpleConfig *plo, AutoStruct *aut) :  KplItem(plo)
{
  char s[256];

  symb = plo->readNumEntry("symb", aut->autoSymb);
  strcpy(s, plo->readEntry("color", ""));
  if (strlen(s))
    sscanf(s, "%lx", &color);
  else
    color = aut->colData;
  fx = plo->readDoubleNumEntry("fx", 1.0);
  fy = plo->readDoubleNumEntry("fy", 1.0);
}

ScaledItem::~ScaledItem()
{
}

void ScaledItem::writePlo(FILE *f)
{
  KplItem::writePlo(f);
  fprintf(f, "symb=%i\ncolor=%lx\nfx=%g\nfy=%g\n",
          symb, color, fx, fy);
}

void ScaledItem::normalize(double fx0, double fy0)
{
  fx = fx0;
  fy = fy0;
}

void ScaledItem::setProperties(KplGraph *g)
{
  g->setColData(color);
  g->setSymbol(symb);
  g->fxn = fx;
  g->fyn = fy;
}
